﻿using Kivii.DataAnnotations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Lims.Entities
{
    /// <summary>
    /// 检验项目类型
    /// </summary>
    public enum ReportItemType
    {
        /// <summary>
        /// 不支持的
        /// </summary>
        Unsupported = 0,

        /// <summary>
        /// 目录类型
        /// </summary>
        Folder=1,

        /// <summary>
        /// 检测项目录,有子项的大项
        /// </summary>
        TestFolder=2,

        /// <summary>
        /// 检测项，子项或者无子项的大项
        /// </summary>
        Test=3,

        /// <summary>
        /// 样品自身属性，如色号
        /// </summary>
        SampleSelf=4,

        /// <summary>
        /// 样品的部分属性,如部位
        /// </summary>
        SamplePart=5
    }

    public enum ReportItemStatusType
    {
        Unsupported = 0,
        //待检测
        BeforeTest = 100,
        //分派中
        Asigning = 150,
        /// <summary>
        /// 正在测试？
        /// </summary>
        //检测中
        Testing = 200,
        //待复核
        BeforeReview = 250,
        //已复核
        TestFinished = 300,
        /// <summary>
        /// 检测结束 
        /// </summary>
        TestCollected = int.MaxValue
    }
    public abstract class ReportItemBase : EntityWithMetadata
    {
        #region 第三方关联，关联业务
        [ApiMember(Description = "业务Kvid")]
        [IgnoreUpdate]
        [DefaultEmptyGuid]
        public Guid BizKvid { get; set; }

        [ApiMember(Description = "业务流水号")]
        [IgnoreUpdate]
        [StringLength(200), Default("")]
        public string BizId { get; set; }

        [ApiMember(Description = "业务类型,指实体类型全名称")]
        [IgnoreUpdate]
        [StringLength(200), Default("")]
        public string BizType { get; set; }
        #endregion

        #region 关联信息，关联报告，关联样品
        /// <summary>
        /// 当RootKvid为空，说明是原生的ReportItem
        /// 否则则说明该ReportItem是引用生成的记录
        /// 便于验证部分字段读写权限
        /// 只有产品标准、标准值可修改，其他字段不可修改
        /// </summary>
        [ApiMember(Description = "根Kvid")]
        [InternalSetter]
        [DefaultEmptyGuid]
        public Guid RootKvid { get; set; }
        /// <summary>
        /// 叛逆的报告Kvid
        /// </summary>
        [ApiMember(Description = "报告Kvid")]
        [Required]
        [IgnoreUpdate]
        [NonClusteredIndex]
        public Guid ReportKvid { get; set; }
        /// <summary>
        /// 样品编号，可以是子样品编号
        /// </summary>
        [ApiMember(Description = "样品Kvid")]
        [Required]
        [IgnoreUpdate]
        public Guid SampleKvid { get; set; }
        /// <summary>
        /// 样品根Kvid
        /// </summary>
        [ApiMember(Description = "样品根Kvid")]
        [DefaultEmptyGuid]
        public Guid SampleRootKvid { get; set; }
        #endregion

        #region 关联标准，检验项目，引用的方法标准中的项目
        /// <summary>
        /// 引用标准Kvid
        /// </summary>
        [ApiMember(Description = "引用标准Kvid")]
        [DefaultEmptyGuid]
        public Guid StandardKvid { get; set; }
        /// <summary>
        /// 标准代码
        /// </summary>
        [ApiMember(Description = "标准代码")]
        [Default("")]
        [StringLength(200)]
        public string StandardCode { get; set; }
        /// <summary>
        /// 标准名称
        /// </summary>
        [ApiMember(Description = "标准名称")]
        [Default("")]
        [StringLength(200)]
        public string StandardTitle { get; set; }
        /// <summary>
        /// 引用检测项目Kvid
        /// </summary>
        [ApiMember(Description = "引用检测项目Kvid")]
        [DefaultEmptyGuid]
        public Guid DetectionKvid { get; set; }
        /// <summary>
        /// 引用检测项目名称
        /// </summary>
        [ApiMember(Description = "引用检测项目名称")]
        [Default("")]
        [StringLength(200)]
        public string DetectionTitle { get; set; }
        /// <summary>
        /// 引用方法标准检测项目Kvid
        /// </summary>
        [ApiMember(Description = "引用方法标准检测项目Kvid")]
        [DefaultEmptyGuid]
        public Guid DetectionReferenceKvid { get; set; }
        /// <summary>
        /// 测试方法
        /// </summary>
        [ApiMember(Description = "测试方法")]
        [Default("")]
        [StringLength(500)]
        public string DetectionCodeActual { get; set; }
        /// <summary>
        /// 标准值数值单位
        /// </summary>
        [Default("")]
        [ApiMember(Description = "数值单位")]
        [StringLength(100)]
        public string DetectionUnit { get; set; }
        /// <summary>
        /// 标准值
        /// </summary>
        [ApiMember(Description = "标准值")]
        [StringLength(2000)]
        public string DetectionValueQualified { get; set; }
        #endregion

        #region 金额相关
        /// <summary>
        /// 货币单位
        /// </summary>
        [ApiMember(Description = "货币单位")]
        [IgnoreUpdate]
        [Default("")]
        [StringLength(20)]
        public CurrencyUnit Currency { get; set; }
        /// <summary>
        /// 服务Kvid
        /// </summary>
        [ApiMember(Description = "服务Kvid")]
        [DefaultEmptyGuid]
        public Guid GoodsKvid { get; set; }
        /// <summary>
        /// 包装Kvid
        /// </summary>
        [ApiMember(Description = "包装Kvid")]
        [DefaultEmptyGuid]
        public Guid PackageKvid { get; set; }
        /// <summary>
        /// 价格类型
        /// </summary>
        [ApiMember(Description = "价格类型")]
        [Default("")]
        public string PackageUnit { get; set; }
        /// <summary>
        /// 理论收费
        /// </summary>
        [ApiMember(Description = "理论收费")]
        [DecimalLength(10, 2), Default(0)]
        public decimal AmountPlan { get; set; }
        /// <summary>
        /// 实际收费
        /// </summary>
        [ApiMember(Description = "实际收费")]
        [DecimalLength(10, 2), Default(0)]
        public decimal Amount { get; set; }
        #endregion
 
        #region 检测Test相关
        /// <summary>
        /// 关联Test
        /// </summary>
        [ApiMember(Description = "关联Test")]
        [DefaultEmptyGuid]
        [InternalSetter]
        public Guid TestKvid { get; set; }
        /// <summary>
        /// 关联TestRoot
        /// </summary>
        [ApiMember(Description = "关联TestRoot")]
        [DefaultEmptyGuid]
        [InternalSetter]
        public Guid TestRootKvid { get; set; }
        /// <summary>
        /// 实测值
        /// </summary>
        [ApiMember(Description = "实测值")]
        [StringLength(500), Default("")]
        [InternalSetter]
        public string TestValue { get; set; }
        /// <summary>
        /// 原始记录路径
        /// </summary>
        [ApiMember(Description = "原始记录路径")]
        [StringLength(200), Default("")]
        [InternalSetter]
        public string TestOriginalFilePath { get; set; }
        /// <summary>
        /// 录入人员Kvid
        /// </summary>
        [ApiMember(Description = "录入人员Kvid")]
        [DefaultEmptyGuid]
        [InternalSetter]
        public Guid TestCreatorKvid { get; set; }
        /// <summary>
        /// 录入人员
        /// </summary>
        [ApiMember(Description = "录入人员")]
        [StringLength(50), Default("")]
        [InternalSetter]
        public string TestCreatorName { get; set; }
        /// <summary>
        /// 审核人员Kvid
        /// </summary>
        [ApiMember(Description = "审核人员Kvid")]
        [DefaultEmptyGuid]
        [InternalSetter]
        public Guid TestReviewerKvid { get; set; }
        /// <summary>
        /// 审核人员
        /// </summary>
        [ApiMember(Description = "审核人员")]
        [StringLength(50), Default("")]
        //[InternalSetter]
        public string TestReviewerName { get; set; }

        /// <summary>
        /// 当Test创建时，代表该次检验结束
        /// </summary>
        [ApiMember(Description = "检测结束时间")]
        public DateTime? TestCreateTime { get; set; }
        /// <summary>
        /// 复核时间
        /// </summary>
        [ApiMember(Description = "复核时间")]
        public DateTime? TestReviewTime { get; set; }
        #endregion

        #region 基本信息
        /// <summary>
        /// 关联父级检Kvid
        /// </summary>
        [ApiMember(Description = "关联父级检Kvid")]
        [DefaultEmptyGuid]
        public Guid ParentKvid { get; set; }
        /// <summary>
        /// 检验项目类型
        /// </summary>
        [ApiMember(Description = "检验项目类型")]
        [StringLength(50)]
        public ReportItemType Type { get; set; }
        /// <summary>
        /// 项目名称
        /// </summary>
        [ApiMember(Description = "项目名称", IsRequired = true)]
        [Required]
        [StringLength(500)]
        public string Title { get; set; }

        /// <summary>
        /// 首要排序号
        /// </summary>
        [ApiMember(Description = "首要排序号")]
        [Default(0)]
        public int SortId { get; set; }

        /// <summary>
        /// 次要排序号
        /// </summary>
        [ApiMember(Description = "次要排序号")]
        [Default(0)]
        public int SortIdEx { get; set; }
        /// <summary>
        /// 判定结果
        /// </summary>
        [ApiMember(Description = "判定")]
        [StringLength(200), Default("")]
        public string Judgement { get; set; }
        /// <summary>
        /// 备注
        /// </summary>
        [ApiMember(Description = "备注")]
        [StringLength(2000)]
        public string Remark { get; set; }
        #endregion

        #region 检测人员信息
        /// <summary>
        /// 录入人员与检测人员的区别：
        /// 录入人员指实际录数据的人员
        /// 检测人员指实验还未开始，由相关部门将该检验分配给的指定检测人员
        /// </summary>

        [ApiMember(Description = "检测小组Kvid")]
        [DefaultEmptyGuid]
        public Guid WorkGroupKvid { get; set; }
        /// <summary>
        /// 检测小组名称
        /// </summary>
        [ApiMember(Description = "检测小组")]
        [StringLength(50)]
        public string WorkGroupName { get; set; }
        /// <summary>
        /// 检测人员Kvid
        /// </summary>
        [ApiMember(Description = "检测人员Kvid")]
        [DefaultEmptyGuid]
        public Guid WorkerKvid { get; set; }
        /// <summary>
        /// 检测人员名称
        /// </summary>
        [ApiMember(Description = "检测人员")]
        [StringLength(50), Default("")]
        public string WorkerName { get; set; }
        #endregion

        #region 创建/更新人V1.0
        [ApiMember(Description = "创建人Kvid ")]
        [IgnoreUpdate]
        [CurrentMemberKvid]
        public Guid CreatorKvid { get; set; }

        [ApiMember(Description = "创建人")]
        [IgnoreUpdate]
        [StringLength(50), CurrentMemberName]
        public string CreatorName { get; set; }

        [ApiMember(Description = "更新人Kvid ")]
        [CurrentMemberKvid]
        public Guid UpdaterKvid { get; set; }

        [ApiMember(Description = "更新人")]
        [StringLength(50), CurrentMemberName]
        public string UpdaterName { get; set; }
        #endregion

        #region 其它不在数据库中的信息
        /// <summary>
        /// 用于查询 接口需要合并部位数据到大项中 返回使用，不记录在数据库中
        /// </summary>
        [Ignore]
        public string TitlePart { get; set; }

        /// <summary>
        /// 用于查询 接口需要合并色号数据到大项中 返回使用，不记录在数据库中
        /// </summary>
        [Ignore]
        public string TitleSelf { get; set; }

        /// <summary>
        /// 用于查询 接口需要合并部位数据到大项中 返回使用，不记录在数据库中
        /// </summary>
        [Ignore]
        public Guid PartKvid { get; set; }
        [Ignore]
        public ReportItemStatusType StatusType
        { 
            get 
            {
                return (ReportItemStatusType)Status;
            }
            set
            {
                Status = (int)value;
            }
        }
        /// <summary>
        /// 子项，不在数据库中
        /// </summary>
        [Ignore]
        public List<ReportItem> SubItems { get; set; }

        #endregion
    }

    [Alias(Configs.TableNameReportItem)]
    [Api(Description = "检验项目")]
    public class ReportItem : ReportItemBase, IEntityInAssemblyDb
    {

    }

    [Alias(Configs.TableNameReportItemHistory)]
    [Api(Description = "归档报告")]
    public class ReportItemHistory : ReportItemBase, IEntityInAssemblyDb
    {

    }

    public class ReportItemEx : ReportItem
    {
        public string ReportId { get; set; }

        public int ReportStatus { get; set; }
        public string ReportDemanderName { get; set; }
        public string ReportSampleManufacturerName { get; set; }
        public string ReportTestDetections { get; set; }
        public decimal? ReportAmountTestPlan { get; set; }
        public decimal? ReportAmountTest { get; set; }
        public decimal? ReportAmount { get; set; }

        //以下用于委托报告不合格项目导出
        public string ReportSampleName { get; set; }
        public string ReportSampleBatchNumber { get; set; }
        //工艺参数，是metadata字段
        //
        public string ReportTestStandards { get; set; }
        public string ReportRemark { get; set; }
        public DateTime? ReportDealDate { get; set; }
        public DateTime? ReportDeadDate { get; set; }
        public string ReportSampleBrand { get; set; }
        public string ReportPayerName { get; set; }
        public string ReportMetadataString { get; set; }

        //以下用于监督报告统计
        public string SamplingOrderID { get; set; }

        public List<ReportItemEx> SubItemExs { get; set; }

    }
}
